--- 2008 SRC "stereo.vhd" VHDL Code 
--- Current file name: stereo.vhd
--- Last Revised:  10/27/2008; 9:45 a.m.
--- Author:  WDR
--- Copyright: William D. Richard, Ph.D.

LIBRARY IEEE ;
USE IEEE.STD_LOGIC_1164.ALL ;
USE IEEE.STD_LOGIC_ARITH.ALL;

ENTITY stereo IS
   PORT (d        : INOUT STD_LOGIC_VECTOR(31 DOWNTO 0) ;
         address  : IN    STD_LOGIC_VECTOR(31 DOWNTO 0) ;
         read     : IN    STD_LOGIC ;
         write    : IN    STD_LOGIC ;
         reset_l  : IN    STD_LOGIC ;
         left     : OUT   STD_LOGIC_VECTOR(15 DOWNTO 0) ;
         right    : OUT   STD_LOGIC_VECTOR(15 DOWNTO 0) ;
         clk      : IN    STD_LOGIC) ;
END stereo ;

ARCHITECTURE behavioral OF stereo IS

   SIGNAL counter    : UNSIGNED(5 DOWNTO 0) ;
   SIGNAL audio_osc  : STD_LOGIC ;

BEGIN

   audio_osc <= counter(5) ;

   oscprocess:PROCESS(clk)
   begin
      IF (clk = '1' AND clk'EVENT) THEN
         IF (reset_l = '0') THEN
            counter <= "000000" ;
         ELSE
            counter <= counter + 1 ;
         END IF;
      END IF;
   END PROCESS oscprocess ; 

   writeprocess:PROCESS(clk)
   begin
      IF (clk = '1' AND clk'EVENT) THEN
         IF (address(31 DOWNTO 0) = "11111111111111111111111111111100" AND
            read = '0' AND write = '1') THEN
            left  <= d(31 DOWNTO 16) ;
            right <= d(15 DOWNTO 0) ;
         END IF;
      END IF;
   END PROCESS writeprocess ; 

   readprocess:PROCESS(clk,address,read,write,counter)
   begin
      IF (address(31 DOWNTO 0) = "11111111111111111111111111111000" AND
          read = '1' AND write = '0' AND clk = '0') THEN
         d(31 DOWNTO 0) <= "0000000000000000000000000000000" & audio_osc ;
      else
	 d(31 DOWNTO 0) <= (OTHERS => 'Z') ;
      END IF;
   END PROCESS readprocess ; 

END behavioral ;
